/************************************************************
 * CoSy conferencing system
 *
 * createdirs.c -- create directories (and files) as
 *                 needed for a new install of CoSy
 *
 * Copyright (C) 2001, Alastair JW Mayer
 *
 * This program is free software licensed under the GPL.
 * See the file COPYING for details.
 *
 ************************************************************/

/*
 * This is a fairly quick'n'dirty program to create the directories
 * that CoSy expects.  This could be done with a script but this
 * program uses the "pathname" stuff that the rest of CoSy uses so
 * that any changes made there will automatically be reflected here.
 * Or so it is hoped.
 *
 * This first version expects that the CoSy root level directory
 * will be the same for all paths (it doesn't really have to be)
 * and that that root is passed as the single command line argument
 * (which should match what's in "pathnames.h".)
 *  e.g.    createdirs /CoSy
 */

#include <stdio.h>
#include <errno.h>

#include "pathnames.h"


main(int argc, char* argv[])
{

	char *cosyroot;

	if (argc == 2) {
		cosyroot = argv[1];
	}
	else {
		fprintf(stderr, "usage: %s cosy_root_dir_name\n", argv[0]);
		exit(0);
	}

	if (makedir(cosyroot))	exit(1);
	if (makedir(UPATH))		exit(1);
	if (makedir(CPATH))		exit(1);
	if (makedir(HPATH))		exit(1);
	if (makedir(MPATH))		exit(1);
	if (makedir(MSPLPATH))	exit(1);
	if (makedir(MTMPATH))	exit(1);
	if (makedir(LOGFILE))	exit(1);

	/* now make the subdirs for confs and users */
	makesubdirs(UPATH);
	makesubdirs(CPATH);

	exit(0);
}


/* the following creates the single-letter subdirectories for
 * users and confs
 */
int makesubdirs(const char* path)
{
#define MAXPATH 100
	char pathname[MAXPATH];
	char ch;
	int  last;

	strncpy(pathname, path, MAXPATH);
	last = strlen(pathname);
	if (last > MAXPATH-2) {
		fprintf(stderr, "path '%s' too long\n", UPATH);
		exit(1);
	}

	if (pathname[last-1] != '/')
		pathname[last++] = '/';
	pathname[last] = 'a';
	pathname[last+1] = '\0';

	for (ch = 'a'; ch <= 'z'; ch++) {
		pathname[last] = ch;
		if (makedir(pathname)) exit(1);
	}
	return 0;
}


int makedir(const char* pathname)
{
	if (mkdir(pathname, 0755)) {
		/* check the error */
		if (errno == EEXIST) {
			/* this might be OK, so we'll ignore it */
			return 0;
		}
		else {
			/* no other errors are acceptable */
			perror(pathname);
			return -1;
		}
	}
	/* if we get here we're okay */
	return 0;
}


