/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/****************************************************************
*  COSY COnference SYstem, Mark 2                 *
*                                       *
*  Author: Alastair J.W. Mayer                              *
****************************************************************/
#define DEFS3_H 1

#include <ctype.h>
/* miscellaneous constants */
/* N.B. MAXINT below should really by MAXLONG - is what its used for */
#define MAXINT 017777777777
#define OFF 0
#define ON  1
#define YES 1
#define NO  0
#define EOS '\0'
#define NL  '\n'
#define TMAX 600
#define LMAX 30

/** pathnames and filenames moved to a separate include file to
    simplify changes      --  A.M. 13/05/85     **/
#include "pathnames.h"
#include "mkpath.h"
/** constants following are for 'mkpath'. Note that it is important
   that MIN_CK < MAX_CK < MIN_UK < MAX_UK  because of the
  way that 'mkpath' works.                         ****/
#define MIN_CK 100
#define CLIST  100
#define HDR    101
#define ABS    102
#define DIR    103
#define TXT    104
#define CNODE  105
#define TNODE  106
#define MEMBR  107
#define WORKF  108
#define MAX_CK 108
#define MIN_UK 200
#define ULIST  200
#define CONF   201
#define RES    202
#define PROF   203
#define SCRATCH 204
#define NUCONF 205
#define UNODE  206
#define PASSWD 207   /* obsolete */ 
#define MXADDR 208
#define NOTEF  209
#define MEM_IN 210
#define MEM_OUT 211
#define MAX_UK 211
#define HDOC   300
#define MEMSPOOL 301
#define MEM_TMP 302
#define SYSTM  999

 /* bit position for subsystem flags */
#define MAIN_SS     (ushort)1
#define READ_SS     (ushort)2
#define JOIN_SS     (ushort)4
#define SHOW_SS     (ushort)8
#define MAIL_SS     (ushort)16
#define MOD_SS (ushort)32
#define EDIT_SS     (ushort)64
#define DATE_SS	(ushort) 128
#define OPTION_SS (ushort) 256
#define SYSMGR_SS (ushort) 512
#define MEMO_SS     (ushort) 1024
#define M_RD_SS     (ushort) 2048
#define M_SD_SS     (ushort) 4096
#define RD_AD_SS  (ushort) 8192
#define ALL_SS  ((ushort) 0177777)
/* token definitions for scanner/lookup */
#define ERR_K  -1
#define AMBIG_K -2
#define NUMBER_K -3
#define NULL_K 0

#define	MAIN_K	0
#define	HELP_K	1
#define	READ_K	2
#define	JOIN_K	3
#define	SHOW_K	4
#define	BYE_K	5
#define	MAIL_K	6
#define MEMO_K 65
#define	MOD_K	7
#define	EDT_K	8
#define	OPT_K	9
#define	CMDS_K	10
#define	BKWD_K	20
#define	CMNT_K	21
#define	DATE_K	22
#define	FIRST_K	23
#define	LAST_K	24
#define	FWRD_K	25
#define	HEAD_K	255
#define	QUIT_K	26
#define	REFR_K	27
#define	SAY_K	28
#define COPY_K 290
#define	SKIP_K	29
#define	TO_K	291
#define SRCH_K  292
#define	WTHD_K	31
#define	RESN_K	35
#define	MOVE_K	36
#define	LIST_K	40
#define	ALL_K	41
#define	NEW_K	42
#define	PARTICIP_K	43
#define	RESUME_K	44
#define	STAT_K		45
#define	WHO_K	46
#define	HELLO_K	50
#define	DEL_K	61
#define	REPLY_K	62
#define	SEND_K	63
#define LEAVE_K 64
#define ACCEPT_K 65
#define CLEAR_K  66
#define ORIG_K 67
#define	REM_K	71
#define	TERM_K	72
#define	TOPIC_K	73
#define	ADD_K	74
#define CHNG_K  75
#define	PROF_K	81
#define	SCRAT_K	82
#define	ADDR_K	83
#define	SET_K	84
#define WORK_K	85
#define	SYN_K	91
#define	UPLD_K	92
#define	DNLD_K	93
#define	TMNL_K	94
#define	FILE_K	998
#define	PRNT_K	999
#define PASS_K 990
/* the following defines tokens for error handler */
#define BADKEY 1
#define AMBKEY 2
/* ... etc */
#define TABMAX 42
#define NMSIZE  15
#define P_SIZE  10L
#define P_FMT  "%10lx"
#define M_SIZE   5
#define M_FMT  "%5x"

#define ED_V "verbose_ed "
#define ED_Q "quiet_ed "
#include <stdio.h>
#ifndef SYS_TYPES
#include <sys/types.h>
#define SYS_TYPES 1
#endif
/* following kluge is because Berkley <sys/dir.h> #defines DIR,
 * which we also use.  Later version will redefine COSY use of DIR
 */


#define byte unsigned char
#define ushort unsigned int
struct wdtbl {   /* 'vocabulary' table */
   char *wrd;
   ushort ss;
   int  key;
   struct wdtbl *left, *right;
   };

struct tnode {   /* topic node */
   char *name;
   int  count;   /* last message read */
   int  rdonly;  /* YES if text unwritable */
   struct tnode *next;
   };

struct cnode {   /* conference node */
   char *c_name;
   int n_topics;
   struct tnode *first, *last;
   struct cnode *c_next, *c_prev;   /* make doubly linked */
   };

struct ctree {
   struct cnode *ct_first, *ct_last;
   };

/*** STRINGS is the file in which the message strings extracted by
  'msgstr' should go, because this is where 'mesg()' will look.  **/
#define STRINGS "/u/usr/amayer/cosy/h/Strings"
#define LIM 120

typedef struct {
   char txt[LIM];
   int ptr,siz;
   } line_ty;

#include "funtypes.h"
/** following added as needed for ST Megamax C *
/* #define void in */
