/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/*******************************************************
* COSY(tm) Computer-Based Conferencing System          *
* Copyright (C) 1983, 1984, 1985, University of Guelph *
* Written by: Alastair J.W. Mayer                      *
*******************************************************/
/*
 *  COSY Conference System.  Copyright (C) 1983, 1984 Univ. of Guelph
 *  Author: Alastair JW Mayer
 *
 *  Mods: joiner() should return YES or NO depending on whether it
 *        handled the command OK.  This allows join cmds to be called
 *        from main level, with appropriate main level error handling
 *        if cmd not found.  In particular, join_conf returns YES if
 *        it managed to join, NO otherwise.
 */
joiner(wd)     /* perform JOIN functions */
char *wd;
{
 int key;
 char word[NMSIZE];
 char *fmt(), *mesg();

#ifdef DEBUG
 puts("JOIN subsection entered");
#endif
 strcpy(word,wd);
 if (word[0] == '\0')
    scan(word);
 key = lookup(word,JOIN_SS);
 switch (key)
    {
    case HELP_K:
       helper("join");
       break;
    case NULL_K:
       join_1st();
       break;
    case NUMBER_K:
       if (Privilege > 10)      /* sysmgr can join anything */
           return(join_conf(word,""));
       else {
         tout(fmt(mesg("Use 'CONV %s'@to read or reply to conv's."),word));
         tout("\n");
         }
       break;
    case RESN_K:
       j_withdraw();
       quit_conf();
       Subsys = MAIN_SS;
       break;
    case NEW_K:
       tout("Use MODERATE NEW to create a new conference.\n");
       break;
    case MOVE_K:
       j_mv_conf();
       break;
    default:
       return(join_conf(word,""));
    }
    return(YES);    /* assume OK if we got this far */
}

j_withdraw() /* resign from a conference */
 {
  char cname[NMSIZE], tname[NMSIZE];
  struct cnode *cp, *cfind();
  struct tnode *tfind();
  char *mkpath(), *fmt(), *mesg();

#ifdef DEBUG
 printf(" WITHDRAWing from conference\n");
#endif
 scan(cname);
 if (cname[0]=='\0') {
    getlin(0,"Conference name? ");
    scan(cname);
    }
 quit_conf();  /* disconnect from conferences */
 if ((cp=cfind(cname)) == NULL) {
    tout(fmt(mesg("You are not a member of conference '%s'.\n"),cname));
    return;
    }
 else 
    /* see if resigning from just a topic */
    if ((scan(tname) != NULL) &&
        ((lookup(tname,JOIN_SS)==TOPIC_K) || (tfind(tname,cp)!=NULL))) {
       if (cp->n_topics <= 1) {     /* resigning from last can cause probs*/
          tout("Cannot resign from the only topic.\n");
          flush();
          return;
          }
       if (lookup(tname,JOIN_SS) == TOPIC_K)
          while (scan(tname) == NULL)
             getlin(0,"Topic name? ");
       if (StrCmp(tname,"quit") != 0) {    /* OK, didn't quit */
          if (tfind(tname,cp) == NULL) {   /* no such topic */
             tout(fmt("No topic '%s' found.\n",tname));
             return;
             }
          tout(fmt("Resigning from '%s/%s'.\n",cname,tname));
          rmv_tname(cp,tname);
          }
       else {     /* quit, so just return */
          return;
          }
       }
    else {
    tout(fmt(mesg("Resigning from conference '%s'.\n"),cname));
    free_tlist(cp->first);
    rm_cnode(cp);
    filter(mkpath(MEMBR,cname),username);     /* remove member name from list */
    }
 }

free_tlist(tp)  /* dispose of topic_list */
struct tnode *tp;
{
   struct tnode *sp;
   while (tp != NULL) {
      sp = tp->next;
      free(tp);
      tp = sp;
      }
}

rm_cnode(cp)  /* remove cp from current conf list */
struct cnode *cp;
{
   struct cnode *sp;

   if (cp != NULL && (sp = Ct->ct_first) != NULL) {
      if (Ct->ct_first == cp) {
         if (Ct->ct_first == Ct->ct_last)
            Ct->ct_last = NULL;
         Ct->ct_first = cp->c_next;
         free(cp);
         }
      else {
         while (sp->c_next != cp && sp->c_next != NULL)
            sp = sp->c_next;
         sp->c_next = cp->c_next;
         if (Ct->ct_last == cp)
            Ct->ct_last = sp;
         free(cp);
         }
      }
}

join_1st()  /* join 1st active mail or conf */
{
  tout("Checking for conference activity.\n");
  if (join_actv(Mt->ct_first,MAIL_SS) &&
      join_actv(Ct->ct_first,READ_SS)) {
     tout(mesg("No unread messages@in conferences."));
     tout("\n");
     joined_flag = OFF;
     Subsys = MAIN_SS;
     /* return (-1); */
     }
}

join_actv(cpl,ss)  /* join 1st active item on cpl list, enter subsys ss */
struct cnode *cpl; /* KDW 840627 - Split into join_ntop()               */
ushort ss;
 {
   struct cnode *cp;
#ifdef DEBUG
 puts(" JOINing first active conference");
#endif
    cp = cpl;
    while(cp != NULL && join_ntop(cp))
	    cp = cp->c_next;
    if (cp == NULL)
	    return(-1);
    else {
       confnode = cp;
       strcpy(confname,cp->c_name);
       logger("JOIN",confname);
       join_topic(topicname);
       joined_flag = ON;
       Subsys = ss;
       return(0);
       }
 }
 
join_ntop(cp)      /* KDW 840627 Find next new topic in conference cp */
struct cnode *cp;  /* Formerly part of join_actv()    */
{
	struct tnode *tp;
	tp = cp->first;
	while(tp != NULL && new_msgs(cp,tp) < 1)
		tp = tp->next;
	if(tp == NULL)
		return(-1);
	else {
		topicnode = tp;
                strcpy(topicname,tp->name);
		return(0);
	}
}

new_msgs(cp,tp)  /* return num of new messages in cp,tp */
struct cnode *cp;
struct tnode *tp;
{
   int n;
    if (tp != NULL) {
        n = (n_msgs(cp->c_name,tp->name) - tp->count);
        if (n < 0) {
           tp->count += n;
           n = 0;
           }
        return(n);
       }
   else
      return(0);
}


get_topic(tn)  /* connect to a topic within conference */
char *tn;
{
 char tname[NMSIZE];
 struct tnode *tp, *tfind();
 FILE *fp, *fopen();
 char *mkpath();

 tname[0] = '\0';
 if (*tn == '\0') {
    scan(tname);
    if (tname[0]=='\0') {     /** if no topic given, list them and prompt **/
       if (confnode != NULL && confnode->n_topics > 1) {
         lyst_topics(confnode);
         getlin(0,"Topic? ");
         scan(tname);
         }
       if (tname[0]=='\0' && confnode != NULL && confnode->first != NULL)
          strcpy(tname,confnode->first->name);  /** default to 1st listed **/
       if (StrCmp(tname,"quit")==0)   /* then never mind */
          return (-1);
       }
    else if (confnode->n_topics==1 && StrCmp(tname,confnode->first->name)){
       unscan(tname);
       strcpy(tname,confnode->first->name);
       }
    strcpy(tn,tname);  /* to return it */
    }
 else
    strcpy(tname,tn);
 if ((tp=tfind(tname,confnode)) != NULL)    /* expands abbrev. name */
    strcpy(tn,tname);
 if ((fp = fopen(mkpath(TNODE,confname,tname),"r")) == NULL)
    return(YES);
 else {
    fclose(fp);
    return(NO);
    }
}

join_topic(tn)  /* open topic Text file & set up pointers */
char *tn;       /* topic name */
{
 char *mkpath();
 char tnam[NMSIZE];
 FILE *fopen();
 struct tnode *tfind(), *mk_tnode();
 char logmsg[50];
 int i, lpcnt = 5;  /* lpcnt (loop count) prevents endless loops */
 int stat;  /* status returned by gettopic */

    last_m_read = 0;
    if (Subsys == MOD_SS)  /* Don't need to join a topic for moderate */
        return(YES);
    strcpy(tnam,tn);
    while (lpcnt-- && ((stat=get_topic(tnam)) > 0)) {
       tout(fmt(mesg("Topic '%s' not found.\n"),tnam));
       tnam[0] = '\0';
       }
    if (stat < 0)   /* did a quit */
       return (NO);
    if (lpcnt <= 0) {   /** couldnt find a topic **/
       tout(fmt("Unable to join '%s'.\n",confname));
       return(NO);
       }
    strcpy(topicname,tnam);
    tout(fmt(mesg("Joining %s/%s |Joining conference '%s', topic '%s'. "),
            confname, topicname));
    if ((ftopic = fopen(mkpath(TXT,confname,topicname),"r")) == NULL ||
       (fmsgdir = fopen(mkpath(DIR,confname,topicname),"r")) == NULL )
       {
       quit_conf();
       tout(mesg("No messages@in this topic")); tout("\n");
       return(NO);
       }
    else {
       if ((topicnode=tfind(topicname,confnode))==NULL) {
          topicnode = mk_tnode(topicname, 0, 0);
          if (confnode->n_topics == 0 || confnode->last == NULL) 
             confnode->first = confnode->last = topicnode;
          else
             confnode->last = (confnode->last->next = topicnode);
          confnode->n_topics++;
          }
       tout(fmt(mesg("%d new message(s).\n"),new_msgs(confnode,topicnode)));
       if (ck_rd_only(ftopic,topicnode))
          tout(mesg("This topic is$READ ONLY\n"));
       }
    /** now set up the bitmap for read messages **/
    /* FIX: following line had condition i<=TMAX, which meant that */
    /* it was actually going one element beyond the end of ReadList*/
    /* and clobbering whatever was there.  Amazing this didn't cause */
    /* any (known) problems before.  -- AJWM 7/27/99               */
    for (i=0; i<TMAX; i++)
       ReadList[i] = 0; 
    for (i=0; i<=topicnode->count; i++)
       r_mrk_rd(i,YES);
    return(YES);
}

struct cnode *cfind(nm)  /* find cnode pointer in list */
char nm[];
{
   struct cnode *p;

   if (numeric(nm))
      p = Mt->ct_first;
   else
      p = Ct->ct_first;
   while (p != NULL && strcmp(p->c_name,nm) != 0)
      p = p->c_next;
   return(p);

}
struct tnode *tfind(nm,cp)  /* find tnode pointer in list cp */
char *nm;
struct cnode *cp;
{
   struct tnode *p;

   if (cp == NULL)
     return(NULL);
   else {
     p = cp->first;
     while (p != NULL && StrCmp(nm,p->name) !=0)
        p = p->next;
     if (p != NULL) strcpy(nm,p->name);
     return(p);
     }
}
