/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/* validate memo files */
#include <stdio.h>
#include "../defs3.h"
#include "../lock/lock.c"
#include "../lock/panic.c"
#include "memo.h"
#include "mkpath.c"
#include "ishere.c"
#define HASREAD 1
#define CANTFIND 2
#define NOTREAD 3

main(argc, argv)    /* check all entries in a user's in & out baskets */
int argc;
char *argv[];
{
   FILE *fb, *ft, *fopen();
   char bask[BSKSIZ];
   char fname[50], tmpnm[50];
   char tonm[NMSIZE];
   int numb;
   int i, okay;

   if (argc < 2) {
      fputs("usage: vmemo name1 [ name2 ... ] \n",stderr);
      exit(0);
      }
   for (i=1; i<argc; i++) {     /* loop through args */
      strcpy(fname,mkpath(MEM_IN,argv[i]));
      strcpy(tmpnm,fname);
      strcat(tmpnm,".VBK");
      /****** C R I T I C A L -- But shouldn't be anyone else on *****/
      if (lock(fname)) {
         fprintf(stderr,"vmemo: can't lock %s\n",fname);
         continue;   /* next user */
         }
      if (link(fname,tmpnm) != 0) {
         perror("vmemo (out.1)");
         continue;   /* next user */
         }
      unlink(fname);  /* don't forget! */
      if ((ft=fopen(tmpnm,"r"))==NULL) {
         perror("vmemo (out.2)");
         unlock(fname);
         continue;   /* next user */
         }
      if ((fb=fopen(fname,"w"))==NULL) {
         perror("vemo (out.3)");
         unlock(fname);
         fclose(ft);
         continue;   /* next user */
         }

      /* everything opened ok, now process the basket */
      
      printf(" ** checking user  '%s'  Memo_in  **\n",argv[i]);
      while (fread(bask,BSKSIZ,1,ft) > 0) {
         sscanf(bask,"%14s%d",tonm,&numb);
         if (!ishere(tonm)) {
            anomaly(tonm,numb,"dest user not here");
            continue;  /* skip */
            }
         if (numgone(numb)) {
            anomaly(tonm,numb,"missing msg file");
            continue;
            }
         okay=YES;
#ifdef DO_OUTS
         switch (findit(tonm,numb,argv[i])) {
            case HASREAD:
                     okay = YES;
                     if (bask[21] != 'R')
                        anomaly(tonm,numb,"read, not marked");
                     bask[21] = 'R';
                     break;
            case CANTFIND: 
                     okay = NO;
                     anomaly(tonm,numb,"not in basket");
                     break;
            case NOTREAD:
                     okay = YES;
                     if (bask[21] == 'R')
                        anomaly(tonm,numb,"marked but not read");
                     bask[21] = ' ';
                     break;
            }
#endif
         if (okay)
            fwrite(bask,BSKSIZ,1,fb);   /* maybe write it out again */
         }  /* end while */
      fclose(fb);
      fclose(ft);
      unlock(fname);
      /*** E N D   C R I T I C A L ***/
      unlink(tmpnm);
      }  /* end this user */

   printf("vmemo complete.\n");
}

anomaly(usr,nmr,msg)   /* print error message */
char *usr, *msg;
int nmr;
{
   printf("%14s %5d %s\n",usr,nmr,msg);
}

int numgone(nmr)    /* check if memo # nmr exists */
int nmr;
{
   char memname[40];
   char mnum[10];

   sprintf(mnum,"%d",nmr);
   return (access(mkpath(MEMSPOOL,mnum),0));
}


int findit(tnm,nmr,fnm)    /* find nmr from fnm in tnm  */
char *tnm, *fnm;
int nmr;
{
   char ibsk[BSKSIZ];
   char denm[NMSIZE]; 
   int mnum;
   FILE *fp;

    if ((fp=fopen(mkpath(MEM_IN,tnm),"r"))==NULL) {
       perror("vmemo (find)");
       return (CANTFIND);
       }
    while (fread(ibsk,BSKSIZ,1,fp) > 0) {
      sscanf(ibsk,"%14s%d",denm,&mnum);
      if ((mnum==nmr) && (strcmp(fnm,denm)==0))  {  /*found it */
         fclose(fp);
         if (ibsk[21]=='R')
            return(HASREAD);
         else
            return(NOTREAD);
         }
    }
      /* got to here, didnt find it */
      return(CANTFIND);
}
