/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/*******************************************************
* COSY(tm) Computer-Based Conferencing System          *
* Copyright (C) 1983, 1984, University of Guelph       *
* Written by: Alastair J.W. Mayer                      *
*******************************************************/

#include <sys/stat.h>

/* functions for various date conversions */
static int day_tab[2][13] = {
    {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
  };

day_of_year(year, month, day) /* set day of year */
int year, month, day;         /* from month & day */
{
   int i, leap;

   leap = year%4 == 0 && year%100 != 0 || year%400 == 0;
   for (i=1; i<month; i++)
       day += day_tab[leap][i];
   return(day);
}

month_day(year, yearday, pmonth, pday) /* set month, day */
int year, yearday, *pmonth, *pday;     /* from day of year */
{
   int i, leap;

   leap = year%4 == 0 && year%100 !=0 || year%400 == 0;
   for (i=1; yearday>day_tab[leap][i]; i++)
      yearday -= day_tab[leap][i];
   *pmonth = i;
   *pday = yearday;
}
/* above two functions are from Kernighan & Ritchie, pg 104 */
/* above two functions are from Kernighan & Ritchie, pg 104 */
long int get_date(s)  /* inverse of system 'ctime' function */
char *s;
{
    char month[15];
    int dd, yy, mm, t;
    long int nsecs;

    mm = 0;
#ifdef DEBUG
   printf("get date parm='%s'\n",s);
   fflush(stdout);
#endif
    sscanf(s,"%d%[^0123456789]%d",&yy,month,&dd);  /* how often will this be right? */
#if (DEBUG>1)
  printf("yy=%d, mo='%s', dd=%d\n",yy,month,dd);
  fflush(stdout);
#endif
    if (month[0] == '/') /* allow numeric format */
       sscanf(s,"%d%d%d",&yy,&mm,&dd);
    if ((yy <= 31) && (dd > 31)) {
       t = yy;
       yy = dd;
       dd = t;
       }
    if (mm == 0)
       mm = lookup(month,DATE_SS);
    if (mm == NUMBER_K)
       sscanf(month,"%d",&mm);
#if (DEBUG>1)
  printf("yy=%d, mm=%d, dd=%d\n",yy,mm,dd);
  fflush(stdout);
#endif
    if ((mm < 1) || (mm > 12)) {
       tout(" Badly formatted date (see 'help date').\n");
       return(0);
       }
    if (yy < 100)
       yy += 1900;
    dd = day_of_year(yy, mm, dd);
    nsecs = (long) (dd-1) * 24 * 60 * 60 + 5 * 60 * 60;
    yy = yy - 1970;
    dd = (365 * yy) + ((yy + 1)/4);
    nsecs += (long) dd * 24 * 60 * 60;
    return(nsecs);
 }
long int mod_time(name)
char *name;
{
   struct stat stbuf;
   if (stat(name, &stbuf) == -1)
       return(-1L);
   return(stbuf.st_mtime);
}
long int acc_time(name)
char *name;
{
  struct stat stbuf;
  if (stat(name, &stbuf) == -1)
     return(-1L);
  return(stbuf.st_atime);
}
