/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/*******************************************************
* COSY(tm) Computer-Based Conferencing System          *
* Written by: Alastair J.W. Mayer                      *
*******************************************************/
#include <sys/stat.h>

/*   reader.c  -- read subsys parser and most read routines      */

int OrigMsg = 0;  /* previous msg for "original" command (Mk3.1) */
int ViewFlag = NO;

reader(wd)     /* perform READER functions */
char wd[];
{
 char word[NMSIZE];
 char cmd[100];
 int key, val;
 int n;
#ifdef DEBUG
 printf("READER subsection entered\n");
#endif
 strcpy(word,wd);
 if (word[0]=='\0')
    scan(word);
#if (DEBUG>1)
 printf(" -next word was :"); printf("%s",word);
 printf(":\n");
#endif
 key=lookup(word,READ_SS);
 switch (key)
    {
    case HELP_K:
       helper("read");
       break;
    case AMBIG_K:
       error(AMBKEY);
       lstambig(word,READ_SS);
       break;
    case BKWD_K:
       read_bkwd();
       break;
    case CMNT_K:
       if (no_priv(2)) break;
       add_cmnt();
       break;
    case COPY_K:
        if (no_priv(3)) {   /* restrict COPY to moderators - AJWM Sep'85 */
           flush();
           break;
           }
        { char *tmpnam="/tmp/CCXXXXXX"; char *mktemp();
          char fname[20];
          FILE *sav;
          char nm[NMSIZE];
          char sv_conf[NMSIZE], sv_top[NMSIZE];  /* save current conf,topic */
         scan(word);
         if (StrCmp(word,"first") == 0) 
             val = 1;
         else
            if (StrCmp(word,"last") == 0) 
                val = -1;
            else
         sscanf(word, "%d", &val);
         strcpy(fname,tmpnam);
         mktemp(fname);
         sav = Out;
         Out = fopen(fname,"w");
         fprintf(Out,"**COPIED FROM: ");
         if (rd_messages(val)) {
            fclose(Out); Out=sav;
            unlink(fname);
            tout("Nothing copied.\n");
            flush();  /* clear remaining input */
            break;
            }
         fclose(Out); Out = sav;
         scan(nm);
         if (strcmp(nm,"to")==0)
            scan(nm);
        strcpy(sv_conf,confname);     /* save conf & topic names for return*/
        strcpy(sv_top,topicname);
        quit_conf();  /* leave this one */
        if (join_conf(nm,""))
            write_msg(0,fname);
        else
           printf("No message copied.\n");
         unlink(fname);  /* get rid of it */
         /*** now return to this conf ***/
         quit_conf();  /* leave old one */
         join_conf(sv_conf,sv_top);
         break;
         }
    case DATE_K:
       rd_messages(read_date());   /* uniform cmd syntax */
       break;
    case FWRD_K:
       read_fwrd();
       break;
    case HEAD_K:
       read_quick();
       break;
    case QUIT_K:
       quit_conf();
       Subsys = MAIN_SS;
       break;
    case REFR_K:
       read_ref();
       break;
    case RESN_K:
       j_withdraw();
       quit_conf();
       Subsys = MAIN_SS;
       break;
    case SAY_K:
     if (no_priv(2)) break;
       add_msg();
       break;
    case SKIP_K:
       read_skip();
       break;
    case NULL_K:
       read_next();
       break;
    case ALL_K:
       if (topicnode == NULL)
          join_1st();
       while (read_next() == 0)
          fputc('\n',Out);
       break;
    case FIRST_K:
       rd_messages(1);
       break;
    case LAST_K:
       rd_messages(-1);
       break;
    case ORIG_K:
       if (OrigMsg < 1) 
          tout(mesg("This is$not a comment.\n"));
       else
          rd_messages(OrigMsg);
       break;
    case NUMBER_K:
       sscanf(word,"%d",&val);
       rd_messages(val);
       break;
    case SRCH_K:
          search("");
          break;
    case WTHD_K:
       r_withdraw('W');
       break;
    case DNLD_K:
       if (strlen(word) >= 2 || yorn("download",' ')) {
          sprintf(cmd,"%s s %s",Dn_Prtcl,mkpath(SCRATCH,username,""));
          system(cmd);
          }
       break;
    case UPLD_K:
       if (strlen(word) >=2 || yorn("upload",' ')) {
          sprintf(cmd,"%s r %s",Up_Prtcl,mkpath(SCRATCH,username,""));
          system(cmd);
          }
       break;
    default:
       if (StrCmp(word,"again")==0) {
          read_msg(last_m_read,NO,0);
          }
       else if (strcmp(word,"restore")==0 && Privilege > 3) {
          r_withdraw(' ');
          }
       else if (strcmp(word,"view")==0 && Privilege > 3) {
          ViewFlag = YES;
          read_msg(last_m_read,NO,0);
          ViewFlag = NO;
          }
       else if (StrCmp(word,"reply")==0) {
          add_cmnt();     /* make 'reply' undocumented synonym for 'comment' */
          }
       else {
          error(BADKEY);
          lstambig("",READ_SS);
          }
       break;
    }
 if (eoln()) for (n=n_msgs(confname,topicname); n; n--) {
    if (has_read(n))
       continue;
    else
       break;
    }
  if (n == 0) {
    if (Subsys == READ_SS) {
     puts(mesg("No more unread@messages in this topic"));
       puts(mesg("Hit <RETURN> for next@active conf/topic."));
       }
   else
     if (Subsys == MAIL_SS) {
      puts("-----");
      if (! Terse) printf("End of conv. #%s. ",confname);
      printf("Enter REPly, Delete, or LEave.\n");
       }
    }
}

rd_messages(from) /* read 1 or more messages */
int from;
{
  int till, key;
  int n;
  char wd[NMSIZE];

#ifdef DEBUG
  puts(" in 'rd_messages' ");
#endif
  scan(wd);
  key=lookup(wd,READ_SS);
  if (key == TO_K) {
     scan(wd);
     key=lookup(wd,READ_SS);
     switch (key) {
        case DATE_K:  till = read_date(); break; /* uniform cmd syntax */
        case FIRST_K: till = 1; break;
        case LAST_K:  till = -1; break;
        case NUMBER_K: sscanf(wd,"%d",&till); break;
        default: till = from; unscan(wd); /* ? */
        }
     }
  else {
     till = from;
     if (key != NULL_K)
        unscan(wd);
     }
  if (from == -1) from = n_msgs(confname,topicname);
  if (till == -1) till = n_msgs(confname,topicname);
  if (rd_direc == BKWD_K) {
     for (n=from; n>=till; n--) {
         if (read_msg(n,NO,0)) return(-1);
         fputc('\n',Out);
         }
     }
  else {
     for (n=from; n<=till; n++) {
         if (read_msg(n,NO,0)) return(-1);
         fputc('\n',Out);
         }
     }
     return(0);
  }

read_bkwd()  /* read bkwd msgs */
 {
#ifdef DEBUG
 printf(" READing BACKWARD \n");
#endif
 rd_direc = BKWD_K;
 puts(mesg("Read direction is BACKWARD."));
 }

read_fwrd() /* read forward (normal) */
 {
#ifdef DEBUG
 printf(" READing FORWARD\n");
#endif
 rd_direc = FWRD_K;
 puts(mesg("Read direction is FORWARD."));
 }

read_quick()  /* read a bunch of headers */
{

#ifdef DEBUG
  printf(" READing headers only\n");
#endif

  Rd_hdr_flag = ON;
}

int backptr;

struct hdrinfo *read_hdr(offset) /* read msg headers */
/* originally returned long, changed to return structure to simplify
  other changes, such as skipping deleted msgs.  -AJWM, 23jul85 */
long offset;
 {
 long txtsize, clock;
 int ptr1, ptr2, bkptr;
 char name[NMSIZE];
 char *cdate, *ctime();
 char dkey;   /* delete flag and key */
 static struct hdrinfo hdrstuff;

  if (ftopic == NULL)
     join_topic("");
  if (ftopic != NULL)
    {
  fseek(ftopic,offset,0);
  fscanf(ftopic,"%10lx%5x%5x%5x%10lx%c%14c",&txtsize,&ptr1,&ptr2,
                                       &bkptr,&clock,&dkey,name);
  newcomment = ptr1;
  nxtcomment = ptr2;
  OrigMsg = backptr = bkptr;
  deblank(name);
/* now transfer stuff to structure to return it */
  hdrstuff.msg_size = txtsize;
  hdrstuff.msg_cmnt = newcomment;
  hdrstuff.msg_othrcm = nxtcomment;
  hdrstuff.msg_orig = backptr;
  hdrstuff.msg_tstamp = clock;
  hdrstuff.msg_flagch = dkey;
  strcpy(hdrstuff.msg_auth, name);
  return(&hdrstuff);
  }
  else
   return(NULL);
}

prnt_hdr(hdr,n)  /* print info in structure hdr */
struct hdrinfo *hdr;
int n;  /* message number */
{
  time_t clock;
  int bkptr, ptr1, ptr2;
  char name[NMSIZE];
  long txtsize;
  char *cdate, *ctime();

  if (hdr == NULL) {
    puts("prnt_hdr: bad message header");
    return;
    }
  txtsize = hdr->msg_size;
  bkptr = hdr->msg_orig;
  ptr1 = hdr->msg_cmnt;
  ptr2 = hdr->msg_othrcm;
  clock = hdr->msg_tstamp;
  strcpy(name,hdr->msg_auth);
#ifdef UTS
  cdate = ctime(clock);
#else
  cdate = ctime(&clock);
#endif
  tout(mesg("==========\n|==========================\n"));
  if (hdr->msg_flagch == 'X') {          /* hook for distributed conferencing */
     char node[NMSIZE];
     fscanf(ftopic,"%14c",node);
     txtsize -= 14;
     deblank(node);
     tout(fmt( "%s/%s #%d, from %s@%s, %ld chars, %s",confname,topicname,
         n,name,node,txtsize,cdate));
     }
  else if (hdr->msg_flagch == 'N') {      /* hook for 'anonymous' messages */
     tout(fmt( "%s/%s #%d, (anonymous by request), %ld chars, %s",
             confname,topicname,n,txtsize,cdate));
     }
  else {
     tout(fmt("%s/%s #%d, from %s, %ld chars, %s",confname,
               topicname,n,name,txtsize,cdate));
     }
  if (bkptr != 0)
     tout(fmt(mesg("Comment to %d. |This is a comment to message %d.\n"),
             bkptr));
  if (ptr1 != 0)
     tout(mesg("Comment(s). |There is/are comment(s) on this message.\n"));
  if (ptr2 != 0)
     tout(fmt(
     mesg("More refs to %d.|There are additional comments to message %d.\n"),
             bkptr));
  tout(mesg("\n----------\n|--------------------------\n"));
#if 0
  switch (dkey) {   /* if flag, then message was deleted, inicate who by */
     /** note: cases 'X' (external) and 'N' (anonymous) handled above **/
     case 'W': return(-1L);  /* author */
     case 'M': return(-2L);  /* moderator */
     case 'S': return(-3L);  /* system manager */
     default : return(txtsize);  /* not deleted */
     }
    }
  else
    return(0L);
#endif
 }

deblank(str)
char str[];
{
   int i;
   i=0;
   while (str[i] != ' ' && str[i] != '\0') i++;
   str[i] = '\0';
}

read_next()  /* read next sequential msg */
{
 int msgno;
 int undisplayed = YES;
 int n_skipped = 0;
 int status = 0;
#ifdef DEBUG
 printf(" READing next message\n");
#endif
  if (topicnode == NULL)
     return(-1);
 while (undisplayed) {   /* use this loop to skip 'tombstones' of del'd msg */
  switch (rd_direc) {
     case BKWD_K: msgno = last_m_read - 1;
                  if (msgno <= 0) {
                      puts(mesg("No unread msgs in BACKWARD direction."));
                      return(-1);
                      }
                  break;
     case REFR_K:  if (nxtcomment != 0) {
                      msgno = nxtcomment;
                      break;
                      }
                   else {
                      printf("%s #%d.",
                          mesg("No further comments@to message"),backptr);
#ifndef NOTSTICKY
                      puts(" Reading next unread.");
#else
/***** above line was inserted for test, next few lines will be
       ignored.  This is to test very-sticky read reference flag  ***/
                read_fwrd();  /* toggle it back */
                      return(-1);
#endif
                      }
     case FWRD_K: for (msgno = topicnode->count;
               has_read(msgno) && (msgno < (TMAX*NBITS)); msgno++);
                  if (msgno > n_msgs(confname,topicname)) {
                      topicnode->count = n_msgs(confname,topicname);
                      if (Subsys==MAIL_SS && isatty(fileno(Out)))
                          return(-1);
                      puts(mesg("Joining next@active topic."));
/*
 *    KDW 840627 - Try and join next topic in this conference else
 *        join next conference
 */
                      quit_top();
                      if (join_ntop(confnode)) {
/*
 *  AJWM 860226 - Start search for next conf from here, not from top
 */
                          struct cnode *cp;
                          cp = confnode;       /* save it thru quit */
                          quit_conf();         /* leave curr conference */
                          if (join_actv(cp,READ_SS) &&  /* short circuit */
                              join_1st())
                              return(-1);  /* no more */
                          }
                      else
                          join_topic(topicname);
                      if (isatty(fileno(Out)))   /* only if a tty */
                          return(-1);           /* stop 'read all' */
                      msgno = topicnode->count + 1;
                      }
                  else {
                   topicnode->count = msgno;
               }
            break;
      }
  status = read_msg(msgno,YES,n_skipped);
  undisplayed = (status == 2);   /* 2 if del'd */
  if (undisplayed) ++n_skipped;
  }
  return(status);
}

read_ref()   /* set up for reading comment chain */
{
#ifdef DEBUG
 printf(" READing message comments.\n");
#endif
 if (newcomment != 0) {
    read_msg(newcomment,NO,0);
    rd_direc = REFR_K;
    printf("%s REFERENCE.\n",mesg("Read@direction is by"));
    }
 else
    puts(mesg("No comments@to message"));
}

read_msg(n,skp_d,nskp)  /* read msg no. n */
int n;
int skp_d;   /* YES to skip deleted msgs */
int nskp;   /* number skipped up to now */
{
 long size, offset;
 long ftell();
 struct hdrinfo *msghdr, *read_hdr();
 int rval;  /* return value to pass back */
#ifdef DEBUG
 printf("%s%d\n"," READing message no. ",n);
#endif
  if (confnode == NULL) {
     puts(mesg("Not joined@to any conference"));
     return(-1);
     }
  if (ftopic == NULL)
       join_topic("");
  if (ftopic != NULL) {
    if (n == -1) {       /* read last message */
       n = n_msgs(confname,topicname);
       }
    else {
       if (n <= 0 || n_msgs(confname,topicname) < n) {
         printf(mesg("No such message number %d.\n"),n);
         return(-1);
         }
       else {
         fseek(fmsgdir,(long)(P_SIZE * (n-1)),0);
         fscanf(fmsgdir,P_FMT,&offset);
         }
       }
    msghdr = read_hdr(offset);
    size = msghdr->msg_size;
    if (ViewFlag) msghdr->msg_flagch = ' ';
    switch (msghdr->msg_flagch) {
       case 'M':
       case 'W':
       case 'S': if (skp_d) {
                    last_m_read = n;
                    r_mrk_rd(n,YES);
                    return(2);
                    }
                 break;
       }
/*
    if ((skp_d) && (nskp > 0)) {
       printf("[skipped %d deleted/withdrawn msgs]\n",nskp);
       }
*/
    prnt_hdr(msghdr,n);
    rval = 0;
   /** have already printed hdr, mark it as read in case of interrupt */
    last_m_read = n;
    r_mrk_rd(n,YES);
    switch (msghdr->msg_flagch) {
       case 'M': tout(" ** Deleted by conference moderator **\n");
                 break;
       case 'S': tout(" ** Deleted by system manager **\n");
                 break;
       case 'W': tout(" ** Withdrawn by author **\n");
                 break;
       default:  if (Rd_hdr_flag) 
                    rval = copyln(ftopic,Out);
                 else 
                    rval = textcopy(ftopic,Out,size);
                 break;
        }
    if (rval==-1) rval = 1;  /* invert sense */
        /* temp kluge to make skip to n work with more.. prompt --
          have read_msg return > 0 if stopped this way (succesful read) */
    fscanf(ftopic,P_FMT,&size); /* chomp! */
    return(rval);
  }
  else return(-1);
}


add_msg()  /* add a new message */
{

#ifdef DEBUG
 printf(" adding NEW MESSAGE to conference\n");
#endif
 if (confnode == NULL) {
   puts(mesg("Not joined@to any conference"));
   return(1);
   }
 if (topicnode->rdonly && (is_mod(username,confname) < 0)) {
    tout(mesg("Can't add msg to$Read-Only topic.\n"));
    return(1);
    }
 fill_scratch((strcmp(ED_V,Editor)==0),YES);
 add_parse(0);
  return(0);
}

add_cmnt()  /* add a comment to message */
{
  char word[20];
  int val, key;
#ifdef DEBUG
 printf(" adding COMMENT to message\n");
#endif
 if (confnode==NULL) {
    puts(mesg("Not joined@to any conference"));
    return;
    }
 if (topicnode->rdonly && (is_mod(username,confname) < 0)) {
    tout(mesg("Can't add comment to$Read-Only topic.\n"));
    return;
    }
 if (last_m_read == 0) {
    if (scan(word) == NULL) {
       tout("Warning: You haven't just-read a message in this topic.\n");
       getlin(0,"Comment to what msg number?");
       scan(word);
       }
     key = lookup(word,READ_SS);
     switch (key) {
      case DATE_K:  val = read_date(); break; /* allow 'comment date dmy' */
      case FIRST_K: val = 1; break;
      case LAST_K:  val = n_msgs(confname,topicname); break;
      case NUMBER_K: sscanf(word,"%d",&val); break;
      default: tout("Number expected, 0 assumed");
               val = 0;
        }
    if (val > n_msgs(confname,topicname)) {
       tout("Not that many messages here!\n");
       val = 0;
       return;
       }
    last_m_read = val;
    }
 printf(mesg("Comment to message number %d. "),last_m_read);
 fill_scratch((strcmp(ED_V,Editor)==0),NO);
 add_parse(last_m_read);
}

quit_conf()  /* leave current conf after leaving current topic */
{            /* KDW June 29/84 */
#ifdef DEBUG
  printf("QUITting this conference \n");
#endif

  joined_flag = OFF;
  quit_top();
  strcpy(confname,"");
  logger("QUIT","");
  confnode = NULL;
}

quit_top()  /* Leave current topic formerly part of quitconf KDW 840627 */
{
  int n;
#ifdef DEBUG
 printf(" QUITting this topic\n");
#endif
 if (topicnode != NULL) {
    for (n=1; has_read(n); n++);
    topicnode->count = n-1;
    }
 if (ftopic != NULL) fclose(ftopic);
 if (fmsgdir != NULL) fclose(fmsgdir);
 ftopic = NULL; fmsgdir = NULL;
 strcpy(topicname,"");
 topicnode = NULL;
 last_m_read = 0;
 rd_direc = FWRD_K;
 put_list(Ct,Mt);  /* update users Confs list (A.M. 21Jun84) */
}

editmsg(caller,callkey)   /* enter editor to create message */
char *caller;
int callkey;
{
 char *fname, *malloc();
#ifdef DEBUG
 printf(" Editing new message or comment.\n");
#endif
  if (callkey==WORKF && strlen(confname)==0) {
     puts(mesg("Not joined@to any conference"));
     return;
     }
 printf(mesg("Editing %s@text"),caller); puts("");
 fname = malloc(80);
 if (strlen(Editor)>0) {
    StrCpy(Editor, fname);
    }
 else
 if (vorq("Verbose or quiet editor")) {
    StrCpy(ED_V,fname);
    }
 else {
    StrCpy(ED_Q,fname);
    if (Terse)
       puts(mesg("(a)ppend, (.) stop input, (w)rite, (q)uit"));
    else {
      puts(mesg("(a)ppend to input text,  (.) to stop input."));
      puts(mesg("(w)rite to save text,    (q)uit to leave editor."));
      }
    }
 if (callkey==WORKF)
    StrCat(mkpath(callkey,confname,""),fname);
 else
    StrCat(mkpath(callkey,username,""),fname);
 fflush(stdout);
 system(fname);
 free(fname);
 puts(mesg("Edit complete."));
 }

long filesize(fname)  /* determine size of fname (if exists) */
char fname[];
{
  struct stat stbuf;

  if (stat(fname, &stbuf) == -1)
     return(-1L);  /* can't find it */
  return(stbuf.st_size);
}

write_msg(mc,fname)     /* write out message */
int mc;
char *fname;
{
FILE *fscr, *ftex, *fopen();
char lname[100];

long  fsize, filesize();
long clock, time();
long  filesize();
long tfsize;
int lstmsgno;

#ifdef DEBUG
 puts(" in write_msg ");
#endif
 if (strlen(fname)==0)
    fname = mkpath(SCRATCH,username,"");
 if ((fsize = filesize(fname)) <= 1) {
    puts(mesg("No message@to add"));
    return(-1);
    }
 tout(mesg("Adding@message.")); fflush(stdout);
 fscr = fopen(fname,"r");
 fname=mkpath(TXT,confname,topicname);
 strcpy(lname,fname);
/*********** critical section *************/
 if (lock(lname)) {
    puts(mesg("Unable to hold TEXT file: contact system manager."));
    if (fscr != NULL) fclose(fscr);
    return(-1);
    }
 if ((ftex = fopen(lname,"a"))==NULL) {
    unlock(lname);
    printf("Can't add\n");
    if (fscr != NULL) fclose(fscr);
    return(-1);
    }
 if ((tfsize = filesize(lname)) == -1)  /* text file size */
    tfsize = 0;  /* 0 for new file */
 fprintf(ftex," %9lx",fsize);
 fprintf(ftex," %4x",0);  /* left-pointer */
 fprintf(ftex," %4x",0);  /* right-pointer */
 fprintf(ftex," %4x",mc);  /* back-pointer */
 clock=time(0);
 fprintf(ftex," %9lx",clock); /* creation time and date */
 fprintf(ftex," %-14s",username);
 filecopy(fscr,ftex);
 fprintf(ftex," %9lx",fsize); /* file size again */
 fclose(ftex);
 fclose(fscr);
 sync();  /* make sure file gets written before we try update ptr */
 if ((fscr = fopen(mkpath(DIR,confname,topicname),"a")) != NULL) {
    fprintf(fscr," %9lx",tfsize);  /* update directory */
     /* Format P_FMT chged to " %9lx" in order to leave space at end
      * of old (11/34) Dir numbers.  Otherwise 'scanf' got confused */
      fseek(fscr,0L,2);  /* try seek to EOF to synch ftell */
    lstmsgno = (int) (ftell(fscr) / P_SIZE); /* now tfsize == last msgno */
    fclose(fscr);
    }
 if (mc > 0)
    update_ptr(mc,lstmsgno);
 unlock(lname);
/******** end critical section **********/
 printf(mesg("..Message %d added.\n"),lstmsgno);
 r_mrk_rd(lstmsgno,YES);
 if ((--lstmsgno)==topicnode->count) topicnode->count++;
  /* only set just-added msg to read if its the only new msg */
 return(0);
 }

update_ptr(mc,np)  /* update  message comment pointer - Critical */
int mc, np;
{
  FILE *fp, *fopen();
  long current, ftell(), offset;
  int curr;

#ifdef DEBUG
 printf(" in update_ptr, m=%d, p=%d.\n",mc,np);
#endif
  if (mc >= np) {
     tout(fmt(clip("Can't comment to$%d -- no such msg number.\n"),mc));
     tout("Added as normal message.");
     return;
     }
  if ((fp = fopen(mkpath(TXT,confname,topicname),"r+")) == NULL) {
     puts(mesg("Unable to link comment. Added as normal message."));
     return;
     }
#ifdef STDIOBUG
  setbuf(fp,0);   /* get around stdio bug */
#endif
  fseek(fmsgdir,(long)( P_SIZE * (mc-1L)),0);
  fscanf(fmsgdir,P_FMT,&offset);
  fseek(fp,offset+(long)P_SIZE,0);
  fscanf(fp,M_FMT,&curr);
  current = (long) curr;
  while (current != 0L) {
     fseek(fmsgdir,(long)(P_SIZE * (current-1)),0);
     fscanf(fmsgdir,P_FMT,&offset);
     fseek(fp,offset+(long)(P_SIZE + M_SIZE),0);
     fscanf(fp,M_FMT,&curr);
     current = (long) curr;
     }
  fseek(fp,(long)( -1L * M_SIZE), 1) ;
  fprintf(fp," %4x",np);
  fclose(fp);
  sync(); /* one more time... */
}

