/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/*******************************************************
* COSY(tm) Computer-Based Conferencing System          *
* Written by: Alastair J.W. Mayer                      *
*******************************************************/

lister()     /* perform LIST (now SHOW) functions */
{
 int key;
 char word[NMSIZE];
#ifdef DEBUG
 printf("SHOW subsection entered\n");
#endif
 scan(word);
#if (DEBUG>1)
 printf(" -next word was :"); printf("%s",word);
 printf(":\n");
#endif
 key=lookup(word,SHOW_SS);
 switch (key)
    {
    case HELP_K:
       helper("show");
       break;
    case ALL_K:
       list_all();
       break;
    case MAIL_K:
        show_mail();
        break;
    case NEW_K:
       list_new();
       break;
    case PARTICIP_K:
       list_part("");
       break;
    case RESUME_K:
       list_res();
       break;
    case STAT_K:
       list_status();
       break;
    case WHO_K:
       list_who();
       break;
    case WORK_K:
       if (strlen(confname) == 0)
          tout(mesg("Not joined.\n|You aren't joined to a conference.\n"));
       else
          flist(mkpath(WORKF,confname,""));
       break;
    case NULL_K:
       list_membr();
       break;
    case SCRAT_K:
       list_scra();
       break;
    case OPT_K:
        show_opt();
        break;
    case AMBIG_K:
       error(AMBKEY);
       lstambig(word,SHOW_SS);
       break;
    default:
       if (StrCmp(word,"groups")==0)
          show_grps();
       else
       list_conf(word);
       break;
    }
}
list_all()  /* list all confs */
 {
 char *clp;
 char name [NMSIZE];
  char msg[150];
 FILE *fp, *popen();

#ifdef DEBUG
 printf(" LISTing ALL \n");
#endif
 scan(name);
 if (strlen(name) == 0) {
    clp = mkpath(CLIST,"","");
    if (flist(clp)) {
       tout(mesg("No conferences in system.\n"));
       }
    else {
        tout(mesg("(o=open, c=closed)\n"));
         }
    }
   else {
      strcpy(msg,"fgrep -i ");
      strcat(msg,name);
      strcat(msg," ");
      strcat(msg,mkpath(CLIST,"",""));
      fflush(stdout);
      if ((fp = popen(msg,"r")) != NULL) {
         textcopy(fp,Out,MAXINT);
         if (pclose(fp)) {
            tout(fmt("%s '%s'.\n",mesg("No info for@conference"),name));
            }
         else {
            tout(mesg("(o=open, c=closed)\n"));
            }
         }
   }
}

list_new()  /* list confs with new activity */
 {
  int listed, prtc;
  int n;
  char *cn;
  struct tnode *tp;
  struct cnode *cp;
  char conftop[30];  /* used for formatting */

#ifdef DEBUG
 printf(" LISTing confs with NEW activity \n");
#endif
   listed = NO;
   cp = Ct->ct_first;
   prtc = NO;
   while (cp != NULL) {           /* for all conferences */
      tp = cp->first;
      cn = cp->c_name;
      while (tp != NULL){         /* for all topics in this conf */
         n = n_msgs(cn,tp->name);
         if ((n = n_msgs(cn,tp->name)) > tp->count) {
            if ( ! prtc) {
               tout(mesg("Conf/Topic               New Messages\n"));
               prtc = YES;
               }
            sprintf(conftop,"%s/%s",cn,tp->name);
            listed = YES;
            if (tout(fmt("%-25s %4d\n",conftop,(n - tp->count))) == NO) {
               tp = NULL;
               cp = NULL; /* ends loop */
               break;
               }
            }
         tp = tp->next;
         }
      if (cp!=NULL) cp = cp->c_next;
      }
   if (listed == NO)
      tout(mesg("No new messages.\n"));
 }

list_res() /* list participant resume(s) */
 {
  char name[NMSIZE];
  char *urp;
#ifdef DEBUG
 printf(" LISTing participant RESUME\n");
#endif
   scan(name);
   urp = mkpath(RES,name,"");
   if (flist(urp))  /* returns 1 if not found */
      tout(fmt(mesg("No resume for participant '%s'.\n"),name));
 }

list_conf(wd) /* list details of conference */
 char wd[];       /* probably, but not necessarily, conference name */
 {
 char *chp;
#ifdef DEBUG
 printf(" LISTing details of conference '%s'.\n",wd);
#endif
   chp = mkpath(HDR,wd,"");
   if (lst_hdr(chp))
      tout(fmt(mesg("No details for conference '%s'.\n"),wd));
 }

list_membr() /* list confs of which you are member */
 {
 struct cnode *cp;
 register int cont = 1;
#ifdef DEBUG
 printf(" LISTing conferences of which you are a member\n");
#endif
   cp = Ct->ct_first;
   if ((cp = Ct->ct_first) == NULL) {
      tout(mesg("You are not a member of any conferences."));
      tout(mesg("Enter 'show all' for a list of all conferences."));
      }
   else
      while (cp != NULL && cont) {
         cont = tout(mesg(fmt("\nConference: %s\n",cp->c_name)));
         if (cont)
             cont = list_top(cp->first,cp->c_name);
         cp = cp->c_next;
         }
/*
 *  List confname only 
 *
 *   cp = Ct->ct_first;
 *   while (cp != NULL) {
 *      fprintf(Out,"%-15s\n",cp->c_name);
 *   cp = cp->c_next;
 *   }
 */
 }

list_top(tp,cn)  /* list topic names & new messages */
struct tnode *tp;
char *cn;
{
   int nm, nt = 0, cont = 1;

   while (tp != NULL && cont) {
      if ((nt++) == 3) {
         cont = tout("\n");
         nt = 1;
         }
      if (cont)
            {
            nm=n_msgs(cn,tp->name);
           fprintf(Out,"%12s %3d/%-3d  ", tp->name, nm - tp->count, nm);
         }
      tp = tp->next;
      }
   if (cont)
       cont = tout("\n");
   return(cont);
}
 
lyst_topics(cp)  /* list topicnames at node cp */
struct cnode *cp;
{
   struct tnode *tp;

   if (cp != NULL)
      if ((tp = cp->first) != NULL) {
         tout("Topics are: ");
         while (tp != NULL) {
            if ((strlen(tp->name) + 4) >= (LinWidth - ColCnt))
		tout("\n");
	    tout(fmt("'%s', ",tp->name));
            tp = tp->next;
            }
         puts(".");
         }

}

list_status()   /* list status of conference */
{
char cname[NMSIZE];
char unam[NMSIZE];
char *mname;
char *ctime();
time_t ct, mod_time();
FILE *fp, *fopen();
register int cont = 1;

#ifdef DEBUG
 printf(" LISTing STATUS\n");
#endif
  scan(cname);
  if (joined_flag && cname[0]=='\0')
     strcpy(cname,confname);
  if (cfind(cname) == NULL  && Privilege < 4) {  /** only stat own confs **/
     tout(fmt(clip("You are$not a member of '%s'\n"),cname));
     return;
     }
  mname=mkpath(MEMBR,cname,"");
  if ((fp = fopen(mname,"r")) == NULL) {
     fprintf(Out,mesg("No info for '%s'.\n"),cname);
     }
  else
     {
     tout(mesg("Member         Last On\n------         -------\n"));
  while ( fscanf(fp,"%s%*[^\n]",unam) != EOF && cont) {
      ct = mod_time(mkpath(CONF,unam,""));
#ifdef UTS
      cont = tout(fmt("%-15s%s",unam,ctime( ct )));
#else
      cont = tout(fmt("%-15s%s",unam,ctime( &ct )));
#endif
      }
  fclose(fp);
  }
}

list_who()
{
  char name[NMSIZE];
   char msg[150];
  FILE *fp, *popen();

  scan(name);
  sprintf(msg,"fgrep -i '%s' %s",name,mkpath(ULIST,"",""));
  fflush(stdout);
  if ((fp = popen(msg,"r")) != NULL) {
     textcopy(fp,Out,MAXINT);
     if (pclose(fp))
        tout(fmt("Can't find info on '%s'.\n",name));
     }
  else tout("Unable to search user list.\n");
}
