/**************************************************************************
*    CoSy conferencing system, written by Alastair Mayer
*    
*    Copyright (C) 1984 University of Guelph
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
***************************************************************************/

/*******************************************************
* COSY(tm) Computer-Based Conferencing System          *
* Copyright (C) 1983, 1984, University of Guelph       *
* Written by: Alastair J.W. Mayer                      *
*******************************************************/
#include <stdlib.h>

char *strsave(s)       /* save string s somewhere */
char *s;
{
  char *p;

  if   ((p = malloc(strlen(s)+1)) != NULL)
      strcpy(p, s);
  else
      no_mem(); /* panic! */
  return(p);
}

struct tnode *talloc()
{
   return((struct tnode *) malloc(sizeof(struct tnode)));
}

struct cnode *cnalloc()
{
   return((struct cnode *) malloc(sizeof(struct cnode)));
}

struct tnode *mk_tnode(nm, mc)    /* make tnode for topic name nm */
char *nm;
int mc;     /* msg count */
{
   struct tnode *p, *talloc();
   char *strsave();

#if (DEBUG>1)
 printf("  tnode topic '%s'\n",nm);
#endif
   if ((p = talloc())==NULL) /* panic */ no_mem();
   p->name = strsave(nm);
   p->next = NULL;
   p->count = mc;
    return(p);
}

struct cnode *add_cnode(nm)  /* add cnode for conf name nm */
char *nm;
{
   struct cnode *p, *cnalloc();
   char *strsave();

#if (DEBUG>1)
  printf(" loading conf info for '%s'\n",nm);
#endif
   if ((p = cnalloc())==NULL) /* panic */ no_mem();
   p->c_name = strsave(nm);
   p->c_next = NULL;
   p->first = p->last = NULL;
   p->n_topics = 0;
   return(p);
}

add_tnode(cp, tp)  /* add tnode tp to confnode cp */
struct cnode *cp;
struct tnode *tp;
{
   cp->n_topics++ ;
   if (cp->first == NULL)
       cp->first = cp->last = tp;
   else
       cp->last = (cp->last->next = tp);
}

no_mem()  /* panic if malloc can't */
{
   puts("cosy: out of memory while loading confs");
   logger("TERMINATE","out-of-memory");
   logger("LOGOFF","");
   fflush(stdout);
   exit(-1);
}


fill_list(ct,mt,key)  /* fill conflist from key=CONF or NUCONF */
struct ctree *ct, *mt;
int key;
{
   struct cnode *cp, *add_cnode(), *cfind();
   struct tnode *tp, *mk_tnode();
   char name[NMSIZE+2], chcnt[NMSIZE+1];
   char chknam[NMSIZE+1];
   int count, tcount;
   int k;
   FILE *fp, *fcount;
   long int acc_time();
   char fname[80], *mkpath();


   strcpy(fname,mkpath(key,username));
   if ((fp = fopen(fname,"r")) == NULL) {
#ifdef DEBUG
 printf(" fill_list: no file '%s'.\n",fname);
#endif
      return(-1);
      }
/************** critical section *********************/
   lock(fname);
   while (fscanf(fp,"%s %15s",chcnt,name) != EOF) {
CHKCONF:
      strcpy(chknam,name);
      if (name[0] == '+' || name[0] == '-')
         strcpy(chknam,&name[1]);
      if (access(mkpath(CNODE,chknam),0)  /* if non-zero, can't access */
          || (!numeric(chcnt))) {   /* topic count field not numeric */
         /* problem, bad conf name? */
         Conf_err = YES;
fprintf(stderr,"Problem with: %s %s\n",chcnt,name);
         do {
           k = fscanf(fp,"%s %s",name,chcnt);
fprintf(stderr,"scanned: %s %s\n",name,chcnt);
         } while ((k != EOF) && (!numeric(name)));  /* better not have numeric topic */
         if (k != EOF) {
                    tcount = atoi(name);
                    strcpy(name,chcnt);
                    sprintf(chcnt,"%d",tcount);
            goto CHKCONF;
            }
         else {
            break; /* out of main while loop */
            }
         } /* end if */
      else
         count = atoi(chcnt);
      if (numeric(name)) {  /* conversation */
         if (mt->ct_last == NULL)
            mt->ct_first = mt->ct_last = add_cnode(name);
         else {
            mt->ct_last->c_next = add_cnode(name);
            mt->ct_last = mt->ct_last->c_next;
            }
         fscanf(fp,"%14s%d",name,&tcount);
         add_tnode(mt->ct_last, mk_tnode(name,tcount));
         /* Conv has only 1 topic */
         }
      else if (name[0] == '-') {  /* removing topic(s) from conf */
          cp = cfind(&name[1]);   /* strip leading '-' */
          for (k=1; k<=count; k++) {
              fscanf(fp,"%s %s",name,chcnt);
              if (numeric(chcnt)) { /*okay*/
                 tcount = atoi(chcnt);
                 }
              else {                /* oops! */
                 Conf_err = YES;
                 if (numeric(name)) {
                    tcount = atoi(name);
                    strcpy(name,chcnt);
                    sprintf(chcnt,"%d",tcount);
                    }
                 goto CHKCONF;
                 }
              if (cp != NULL)
                  rmv_tname(cp,name);
              }
          }
     else {
         cp = NULL;
         if (name[0] == '+') {    /* add to existing conf */
             cp = cfind(&name[1]);
             if (cp == NULL)
                 strcpy(name,&name[1]);
             }
         if (cp == NULL) {        /* new conf, or + not found */
             if (ct->ct_last == NULL)
                 ct->ct_first = ct->ct_last = cp = add_cnode(name);
             else {
                 ct->ct_last->c_next = cp = add_cnode(name);
                 ct->ct_last = cp;
                 }
             }
         for (k=1; k<=count; k++) {
              fscanf(fp,"%s %s",name,chcnt);
              if (numeric(chcnt)) { /*okay*/
                 tcount = atoi(chcnt);
                 }
              else {                /* oops! */
                 Conf_err = YES;
                 if (numeric(name)) {
                    tcount = atoi(name);
                    strcpy(name,chcnt);
                    sprintf(chcnt,"%s",tcount);
                    }
                 goto CHKCONF;
                 }
             if (access(mkpath(TNODE,cp->c_name,name),0)==0)
                add_tnode(cp, mk_tnode(name,tcount));
             else {
                fprintf(stderr,"Nonexistant: '%s/%s' - discarded\n",
                                cp->c_name,name);
                Conf_err = YES;
                }
             }
         }
    }  /* end while */
   /** before we erase 'Newconfs' we'd better update 'Confs',
    *  in case session aborts.  */
   if (key == NUCONF) {
       put_list(ct,mt);
       /* now zap Newconf */
       fclose(fp);
       fp = NULL;     /* for test below */
       unlink(fname);
       }
   unlock(fname);
/******************** end critical section **********/
   if (fp != NULL) fclose(fp);
   return(0);
 }
put_list(ct,mt)  /* write conflist */
struct ctree *ct, *mt;  /* first in list */
{
   struct cnode *cct, *mmt;
   char *fname;
   char *mkpath();
   FILE *fp, *fcount;

   if (ct->ct_first == NULL && mt->ct_first == NULL)
      return;
   fname = mkpath(CONF,username);
/*************** critical section *********************/
/*                                                    */
   lock(fname);
   if ((fp = fopen(fname,"w")) == NULL) {
      unlock(fname);
      return(-1);
      }
   cct = ct->ct_first;
   while (cct != NULL) {
      fput_clst(fp,cct,NO);
      cct = cct->c_next;
      }
   mmt = mt->ct_first;
   while (mmt != NULL) {
      fput_clst(fp,mmt,NO);
      mmt = mmt->c_next;
      }
   fclose(fp);
   unlock(fname);
/*                                           */
/************ end critical section ***********/
 }

